import configparser
import json
from fastapi import FastAPI, File, UploadFile, HTTPException
from fastapi.responses import FileResponse
from fastapi.middleware.cors import CORSMiddleware
from pydantic import BaseModel
import xmltodict
from fastapi.encoders import jsonable_encoder
import os
import shutil

#FastAPI Backend for UNICADO Gui
app = FastAPI()

origins = [
    "http://localhost",
    "http://localhost:5173",
    "http://localhost:5174",
]

app.add_middleware(
    CORSMiddleware,
    allow_origins=origins,
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

#Models for requests
class Modules(BaseModel):
    groups: list = []

class Conf(BaseModel):
    module_configuration_file: dict = {}


#Endpoints
@app.get("/start/{module}")
async def start_module(module: str):
    #dummy-function
    #waiting for working unicado
    return {"message": f"running {module} without problems"}


#endpoint which returns a list of available modules
@app.get("/modules/selection")
async def get_avail_modules():
    dict = {}
    config = configparser.ConfigParser()
    config.read('xml_configs/config.ini')

    dict['availableModules']=config['available_Modules']['modules'].split(',')
    return dict


#TO-DO: following endpoints are for the case of a single project, need to add solution for more projects
#endpoint which returns the last saved project
@app.get("/modules")
async def get_modules():
    with open("tmp/modules", "r") as fp:
        modules = json.load(fp)

    return modules

#endpoint to save the current project on the server
@app.put("/modules/update")
async def update_modules(item:Modules):
    print(jsonable_encoder(item))
    with open("tmp/modules", "w") as fp:
        json.dump(jsonable_encoder(item), fp)

    return item

#endpoint to get the settings of selected module
@app.get("/modules/{module}/config")
async def get_module_config(module: str):
    with open("xml_configs/"+module+"_conf.xml", encoding='utf-8') as fd:
        config = json.dumps(xmltodict.parse(fd.read()))
        return config

#endpoint to save the setting of selected module
@app.put("/modules/config/update")
async def update_module_config(config: Conf):
    dict = {"module_configuration_file": config.module_configuration_file}
    xml = xmltodict.unparse(dict, pretty=True)
    with open("xml_configs/" + config.module_configuration_file["@name"], "w", encoding='utf-8') as fd:
        fd.write(xml)
        return

#endpoint reset settings of selected module to default
@app.get("/modules/{module}/config/reset")
async def reset_module_config(module:str):
    with open("xml_configs/"+module+"_conf_default.xml", encoding='utf-8') as fd:
        dict = xmltodict.parse(fd.read())
    xml = xmltodict.unparse(dict, pretty=True)
    with open("xml_configs/"+module+"_conf.xml", "w", encoding='utf-8') as fd:
        fd.write(xml)
    return

#dummy endpoints
@app.get("/graph/")
async def get_plotData():
    #dummy-Function
    #TO-DO: working Implemantion
    plotData = {}
    return plotData

@app.get("/log/{file}")
async def get_csv(file: str):
    return FileResponse(f"output_files/logFiles/{file}")

@app.get("/report/{file}")
async def get_report(file: str):
    return FileResponse(f"output_files/reportHTML/{file}")

#location for the porject lists
UPLOAD_DIRECTORY = "./project_lists/"

if not os.path.exists(UPLOAD_DIRECTORY):
    os.makedirs(UPLOAD_DIRECTORY)

#endpoint to save the project list on the server
@app.post("/projects/")
async def upload_project_list(file: UploadFile = File(...)):
    if not file.filename.endswith(".zip"):
        raise HTTPException(status_code=400, detail="Only .zip files are allowed")
    
    file_path = os.path.join(UPLOAD_DIRECTORY, file.filename)

    with open(file_path, "wb") as buffer:
        shutil.copyfileobj(file.file, buffer)

    return {"Project list successfully saved"}

#endpoint to get the project list from the server
@app.get("/projects/")
async def get_project_list():
    file_path = UPLOAD_DIRECTORY + "projects.zip"
    return FileResponse(path=file_path, filename="projects.zip", media_type="application/zip")

#TODO get file per project
#endpoint to get the aircraft_exchange_file
@app.get("/projects/aircraft_exchange_file/")
async def get_aircraft_exchange_file():
   with open("xml_configs/csmr-2020.xml", encoding='utf-8') as fd:
        config = json.dumps(xmltodict.parse(fd.read()))
        return config 