//Interface for Ui-elements in module settings and it's implementations
export interface SettingsUi {
    type: string;
    label: string;
    description: string;
    bind: string;
}

export class SettingsSwitch implements SettingsUi {
    readonly type = "switch";
    label: string;
    description: string;
    bind: string;

    constructor(label: string, description: string, bind: string) {
        this.bind = bind;
        this.label = label;
        this.description = description;
    }
}

export class SettingsSelector implements SettingsUi {
    readonly type = "selector";
    label: string;
    description: string;
    bind: string;
    options: string[];

    constructor(
        label: string,
        description: string,
        bind: string,
        options: string[],
    ) {
        this.bind = bind;
        this.label = label;
        this.description = description;
        this.options = options;
    }
}

export class SettingsTextfield implements SettingsUi {
    readonly type = "textfield";
    label: string;
    description: string;
    bind: string;
    input: string;

    constructor(
        label: string,
        description: string,
        bind: string,
        input: string,
    ) {
        this.bind = bind;
        this.label = label;
        this.description = description;
        this.input = input;
    }
}

export class SettingsNumberfield implements SettingsUi {
    readonly type = "numberfield";
    label: string;
    description: string;
    bind: string;
    unit: string = "-";
    max: number = Number.POSITIVE_INFINITY;
    min: number = Number.NEGATIVE_INFINITY;

    constructor(label: string, description: string, bind: string) {
        this.bind = bind;
        this.label = label;
        this.description = description;
    }
    setMax(max: number) {
        this.max = max;
    }
    setMin(min: number) {
        this.min = min;
    }
    setUnit(unit: string) {
        this.unit = unit;
    }
}

export class SettingsLabel implements SettingsUi{
    readonly type = "label";
    label: string;
    bind: string;
    description: string;

    constructor(label: string,description: string, bind: string) {
        this.label = label;
        this.description = description;
        this.bind = bind;
    }


}
