import {
    SettingsLabel,
    SettingsNumberfield,
    SettingsSelector,
    SettingsSwitch,
    SettingsTextfield, type SettingsUi
} from "$lib/settings/SettingsUi";

//template for datatype of dictionaries, only to avoid errors in Typescript
export type dictionary = {
    [key: string]: any
}

//Test if the string includes a number
export function isNumber(n: string) {
    return !isNaN(parseFloat(n)) && isFinite(parseFloat(n));
}


//Calculate Stepsize of number inputs from current value
export function calcStepSize(value: number) {
    const numberString = value.toString();
    if (!numberString.includes(".")) {
        return 1;
    } else {
        const decimalPart = numberString.split(".")[1];
        return Math.pow(10, -decimalPart.length);
    }
}
//Replacing underscores to space
export function formatLabel(label: string) {
    return label.replaceAll("_", " ");
}

/*Parse json response to matching SettingsUi-Elements
    * to portray it later in the DOM
    * recursive Function
    * */
export function generateHTML(json: dictionary, html : SettingsUi[]) {
    Object.entries(json).forEach(([key, value]) => {
        if (value == null) {
            return
        }
        if (value.hasOwnProperty("value")) {
            if (value["@description"] && value["@description"].toLowerCase().includes("switch")) {
                value.value = (value.value === 'true' || value.value === "1")  //transform string input to boolean for easier binding
                html.push(new SettingsSwitch(key, value["@description"], value))
            }else if(value["@description"] && (value["@description"].toLowerCase().includes("selector:")||value["@description"].toLowerCase().startsWith("selector"))){
                let options = []
                if(value["@description"].includes("mode_" )){
                    options = value["@description"].match(/mode_\d+/g)
                }else{
                    let selectorString = value["@description"].split("Selector:")[1]
                    options=selectorString.split(" / ")
                }
                html.push(new SettingsSelector(key, value["@description"], value, options))
            }else if(isNumber(value.value)){
                value.value=parseFloat(value.value);
                let numberField  = new SettingsNumberfield(key, value["@description"], value)
                if(value.hasOwnProperty('lower_boundary')&&isNumber(value['lower_boundary'])){numberField.setMin(value['lower_boundary'])}
                if(value.hasOwnProperty('upper_boundary')&&isNumber(value['upper_boundary'])){numberField.setMax(value['upper_boundary'])}
                if(value.hasOwnProperty("@unit")){numberField.setUnit(value["@unit"])}//spaeter entweder unit oder Unit in XML
                if(value.hasOwnProperty("@Unit")){numberField.setUnit(value["@Unit"])}
                if(value.hasOwnProperty("unit")){numberField.setUnit(value["unit"])}
                if(value.hasOwnProperty("Unit")){numberField.setUnit(value["Unit"])}
                html.push(numberField);
            } else {
                html.push(new SettingsTextfield(key, value["@description"], value, key))
            }
        } else if (typeof value === "object") {
            html.push(new SettingsLabel(key,'',''))
            generateHTML(json[key],html)
        } else {
            return;
        }
    })
}
