import { writable } from 'svelte/store';

// Define the type for the entire settings data object
/**
 * @typedef {Object<string, string>} SettingsData
 */

export const settingData = writable({});


/**
 * Load XML data into the tab store dynamically.
 * @param {string} data - The XML data as a string.
 * @returns {Promise<void>}
 */
async function loadTabData(data) {
    const parsedXML = JSON.parse(data);
    const root = parsedXML.aircraft_exchange_file;

    /** @type {SettingsData} */
    const tabs = {};

    for (const [key, value] of Object.entries(root)) {
        tabs[key] = value;
    }

    settingData.set(tabs);
}

export { loadTabData };